import psycopg
import time
import os
import logging

DB_USER = os.environ.get("POSTGRES_USERNAME")
DB_PASSWORD = os.environ.get("POSTGRES_PASSWORD")
DB_HOST = os.environ.get("POSTGRESQL_HA_PGPOOL_SERVICE_HOST")
DB_PORT = os.environ.get("POSTGRESQL_HA_PGPOOL_SERVICE_PORT")
DB_NAME = "pace"
POLICY_EPOCH_TABLE_NAME = "policynsxconfigepoch"
BUILD_TAG = os.environ.get("BUILD_TAG")


def wait_until_config_sync_complete(tries=60, sleep_between_tries=15):
    for num_try in range(tries):
        if is_config_sync_complete():
            logging.info("Config sync completed")
            exit(0)
        if num_try < tries - 1:
            logging.info("Waiting %s seconds before retrying check for config sync"
                         % sleep_between_tries)
            time.sleep(sleep_between_tries)
    logging.error("Config sync is not finished after %s seconds" %
                  tries * sleep_between_tries)
    exit(1)


def is_config_sync_complete():
    def test_query_result(rows):
        # expected output is either [(True,)] or [(False,)]
        if rows is None or len(rows) != 1 or len(rows[0]) != 1 or rows[0][0] is not True:
            return False
        return True

    conn = psycopg.connect("dbname={} user={} password={} host={} port={}".format(DB_NAME, DB_USER, DB_PASSWORD, DB_HOST, DB_PORT))
    cursor = conn.cursor()

    try:
        # 1. Policy epoch table is created
        cursor.execute(
            "SELECT EXISTS (SELECT TRUE FROM information_schema.tables WHERE table_schema='public' AND table_name='%s')"
            % POLICY_EPOCH_TABLE_NAME)
        rows = cursor.fetchall()
        if not test_query_result(rows):
            logging.debug("Policy epoch table is not created.")
            return False

        # 2. A published epoch exists in policy epoch table
        cursor.execute("SELECT EXISTS (SELECT TRUE FROM %s WHERE published IS TRUE)"
                       % POLICY_EPOCH_TABLE_NAME)
        rows = cursor.fetchall()
        if not test_query_result(rows):
            logging.debug(
                "A published epoch doesn't exist in policy epoch table.")
            return False

        return True

    finally:
        if cursor:
            cursor.close()
        if conn:
            conn.close()


logging.basicConfig(level=logging.DEBUG)

if BUILD_TAG.startswith("kindtest"):
    logging.info("Skipping config sync check for kind test")
    exit(0)

wait_until_config_sync_complete()
